@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,10 +30,8 @@
 #include "CPU.h"
 #include "LLIntCommon.h"
 #include "MinimumReservedZoneSize.h"
-#include "SigillCrashAnalyzer.h"
 #include <algorithm>
 #include <limits>
-#include <math.h>
 #include <mutex>
 #include <stdlib.h>
 #include <string.h>
@@ -42,8 +40,9 @@
 #include <wtf/DataLog.h>
 #include <wtf/NumberOfCores.h>
 #include <wtf/Optional.h>
-#include <wtf/PointerPreparations.h>
+#include <wtf/OSLogPrintStream.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/TranslatedProcess.h>
 #include <wtf/text/StringBuilder.h>
 #include <wtf/threads/Signals.h>
 
@@ -51,10 +50,6 @@
 #include <crt_externs.h>
 #endif
 
-#if ENABLE(JIT)
-#include "MacroAssembler.h"
-#endif
-
 namespace JSC {
 
 template<typename T>
@@ -225,7 +220,7 @@ static int32_t computePriorityDeltaOfWorkerThreads(int32_t twoCorePriorityDelta,
     return multiCorePriorityDelta;
 }
 
-static bool jitEnabledByDefault()
+static constexpr bool jitEnabledByDefault()
 {
     return is32Bit() || isAddress64Bit();
 }
@@ -235,6 +230,15 @@ static unsigned computeNumberOfGCMarkers(unsigned maxNumberOfGCMarkers)
     return computeNumberOfWorkerThreads(maxNumberOfGCMarkers);
 }
 
+static bool defaultTCSMValue()
+{
+#if CPU(X86_64) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 110000) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED < 140000))
+    return true;
+#else
+    return false;
+#endif
+}
+
 const char* const OptionRange::s_nullRangeStr = "<null>";
 
 bool OptionRange::init(const char* rangeString)
@@ -352,7 +356,7 @@ static void overrideDefaults()
     Options::mediumHeapRAMFraction() = 0.9;
 #endif
 
-#if PLATFORM(IOS_FAMILY) && !PLATFORM(WATCHOS) && defined(__LP64__)
+#if ENABLE(SIGILL_CRASH_ANALYZER)
     Options::useSigillCrashAnalyzer() = true;
 #endif
 
@@ -381,19 +385,26 @@ static void correctOptions()
         Options::thresholdForGlobalLexicalBindingEpoch() = UINT_MAX;
 }
 
-static void recomputeDependentOptions()
+static void disableAllJITOptions()
 {
-#if !defined(NDEBUG)
-    Options::validateDFGExceptionHandling() = true;
-#endif
-#if !ENABLE(JIT)
     Options::useLLInt() = true;
     Options::useJIT() = false;
     Options::useBaselineJIT() = false;
     Options::useDFGJIT() = false;
     Options::useFTLJIT() = false;
+    Options::useBBQJIT() = false;
+    Options::useOMGJIT() = false;
     Options::useDOMJIT() = false;
     Options::useRegExpJIT() = false;
+}
+
+void Options::recomputeDependentOptions()
+{
+#if !defined(NDEBUG)
+    Options::validateDFGExceptionHandling() = true;
+#endif
+#if !ENABLE(JIT)
+    disableAllJITOptions();
 #endif
 #if !ENABLE(CONCURRENT_JS)
     Options::useConcurrentJIT() = false;
@@ -413,14 +424,26 @@ static void recomputeDependentOptions()
     Options::useConcurrentGC() = false;
 #endif
 
+    // At initialization time, we may decide that useJIT should be false for any
+    // number of reasons (including failing to allocate JIT memory), and therefore,
+    // will / should not be able to enable any JIT related services.
     if (!Options::useJIT()) {
+        disableAllJITOptions();
+        Options::useConcurrentJIT() = false;
         Options::useSigillCrashAnalyzer() = false;
         Options::useWebAssembly() = false;
+        Options::usePollingTraps() = true;
     }
 
     if (!jitEnabledByDefault() && !Options::useJIT())
         Options::useLLInt() = true;
 
+    if (WTF::isX86BinaryRunningOnARM() && Options::useJIT()) {
+        Options::useBaselineJIT() = false;
+        Options::useDFGJIT() = false;
+        Options::useFTLJIT() = false;
+    }
+
     if (!Options::useWebAssembly())
         Options::useFastTLSForWasmContext() = false;
     
@@ -468,14 +491,6 @@ static void recomputeDependentOptions()
         Options::maximumEvalCacheableSourceLength() = 150000;
         Options::useConcurrentJIT() = false;
     }
-#if ENABLE(SEPARATED_WX_HEAP)
-    // Override globally for now. Longer term we'll just make the default
-    // be to have this option enabled, and have platforms that don't support
-    // it just silently use a single mapping.
-    Options::useSeparatedWXHeap() = true;
-#else
-    Options::useSeparatedWXHeap() = false;
-#endif
 
     if (Options::alwaysUseShadowChicken())
         Options::maximumInliningDepth() = 1;
@@ -520,6 +535,19 @@ static void recomputeDependentOptions()
         Options::randomIntegrityAuditRate() = 0;
     else if (Options::randomIntegrityAuditRate() > 1.0)
         Options::randomIntegrityAuditRate() = 1.0;
+
+    if (!Options::allowUnsupportedTiers()) {
+#define DISABLE_TIERS(option, flags, ...) do { \
+            if (!Options::option())            \
+                break;                         \
+            if (!(flags & SupportsDFG))        \
+                Options::useDFGJIT() = false;  \
+            if (!(flags & SupportsFTL))        \
+                Options::useFTLJIT() = false;  \
+        } while (false);
+
+        FOR_EACH_JSC_EXPERIMENTAL_OPTION(DISABLE_TIERS);
+    }
 }
 
 inline void* Options::addressOfOption(Options::ID id)
@@ -534,6 +562,14 @@ inline void* Options::addressOfOptionDefault(Options::ID id)
     return reinterpret_cast<uint8_t*>(&g_jscConfig.options) + offset;
 }
 
+#if OS(WINDOWS)
+// FIXME: Use equalLettersIgnoringASCIICase.
+inline bool strncasecmp(const char* str1, const char* str2, size_t n)
+{
+    return _strnicmp(str1, str2, n);
+}
+#endif
+
 void Options::initialize()
 {
     static std::once_flag initializeOptionsOnceFlag;
@@ -541,6 +577,8 @@ void Options::initialize()
     std::call_once(
         initializeOptionsOnceFlag,
         [] {
+            AllowUnfinalizedAccessScope scope;
+
             // Sanity check that options address computation is working.
             RELEASE_ASSERT(Options::addressOfOption(useKernTCSMID) ==  &Options::useKernTCSM());
             RELEASE_ASSERT(Options::addressOfOptionDefault(useKernTCSMID) ==  &Options::useKernTCSMDefault());
@@ -605,14 +643,19 @@ void Options::initialize()
             ASSERT(Options::thresholdForOptimizeAfterWarmUp() >= 0);
             ASSERT(Options::criticalGCMemoryThreshold() > 0.0 && Options::criticalGCMemoryThreshold() < 1.0);
 
-            dumpOptionsIfNeeded();
-            ensureOptionsAreCoherent();
-
 #if HAVE(MACH_EXCEPTIONS)
             if (Options::useMachForExceptions())
                 handleSignalsWithMach();
 #endif
 
+#if OS(DARWIN)
+            if (Options::useOSLog()) {
+                WTF::setDataFile(OSLogPrintStream::open("com.apple.JavaScriptCore", "DataLog", OS_LOG_TYPE_INFO));
+                // Make sure no one jumped here for nefarious reasons...
+                RELEASE_ASSERT(useOSLog());
+            }
+#endif
+
 #if ASAN_ENABLED && OS(LINUX) && ENABLE(WEBASSEMBLY_FAST_MEMORY)
             if (Options::useWebAssemblyFastMemory()) {
                 const char* asanOptions = getenv("ASAN_OPTIONS");
@@ -629,7 +672,12 @@ void Options::initialize()
             Options::dumpZappedCellCrashData() =
                 (hwPhysicalCPUMax() >= 4) && (hwL3CacheSize() >= static_cast<int64_t>(6 * MB));
 #endif
-        });
+
+            // The following should only be done at the end after all options
+            // have been initialized.
+            dumpOptionsIfNeeded();
+            ensureOptionsAreCoherent();
+    });
 }
 
 void Options::dumpOptionsIfNeeded()
@@ -660,6 +708,12 @@ void Options::dumpOptionsIfNeeded()
     }
 }
 
+void Options::finalize()
+{
+    ASSERT(!g_jscConfig.options.allowUnfinalizedAccess);
+    g_jscConfig.options.isFinalized = true;
+}
+
 static bool isSeparator(char c)
 {
     return isASCIISpace(c) || (c == ',');
@@ -667,7 +721,8 @@ static bool isSeparator(char c)
 
 bool Options::setOptions(const char* optionsStr)
 {
-    RELEASE_ASSERT(!g_jscConfig.isPermanentlyFrozen);
+    AllowUnfinalizedAccessScope scope;
+    RELEASE_ASSERT(!g_jscConfig.isPermanentlyFrozen());
     Vector<char*> options;
 
     size_t length = strlen(optionsStr);
@@ -764,8 +819,8 @@ bool Options::setOptionWithoutAlias(const char* arg)
     // if the value makes sense. Otherwise, move on to checking the next option.
 #define SET_OPTION_IF_MATCH(type_, name_, defaultValue_, availability_, description_) \
     if (strlen(#name_) == static_cast<size_t>(equalStr - arg)      \
-        && !strncmp(arg, #name_, equalStr - arg)) {                \
-        if (Availability::availability_ != Availability::Normal     \
+        && !strncasecmp(arg, #name_, equalStr - arg)) {            \
+        if (Availability::availability_ != Availability::Normal    \
             && !isAvailable(name_##ID, Availability::availability_)) \
             return false;                                          \
         Optional<OptionsStorage::type_> value;                     \
@@ -808,7 +863,7 @@ bool Options::setAliasedOption(const char* arg)
     // if the value makes sense. Otherwise, move on to checking the next option.
 #define FOR_EACH_OPTION(aliasedName_, unaliasedName_, equivalence) \
     if (strlen(#aliasedName_) == static_cast<size_t>(equalStr - arg)    \
-        && !strncmp(arg, #aliasedName_, equalStr - arg)) {              \
+        && !strncasecmp(arg, #aliasedName_, equalStr - arg)) {          \
         String unaliasedOption(#unaliasedName_);                        \
         if (equivalence == SameOption)                                  \
             unaliasedOption = unaliasedOption + equalStr;               \
@@ -819,7 +874,7 @@ bool Options::setAliasedOption(const char* arg)
                 return false;                                           \
             unaliasedOption = unaliasedOption + "=" + invertedValueStr; \
         }                                                               \
-        return setOptionWithoutAlias(unaliasedOption.utf8().data());   \
+        return setOptionWithoutAlias(unaliasedOption.utf8().data());    \
     }
 
     FOR_EACH_JSC_ALIASED_OPTION(FOR_EACH_OPTION)
@@ -832,6 +887,7 @@ bool Options::setAliasedOption(const char* arg)
 
 bool Options::setOption(const char* arg)
 {
+    AllowUnfinalizedAccessScope scope;
     bool success = setOptionWithoutAlias(arg);
     if (success)
         return true;
@@ -842,6 +898,7 @@ bool Options::setOption(const char* arg)
 void Options::dumpAllOptions(StringBuilder& builder, DumpLevel level, const char* title,
     const char* separator, const char* optionHeader, const char* optionFooter, DumpDefaultsOption dumpDefaultsOption)
 {
+    AllowUnfinalizedAccessScope scope;
     if (title) {
         builder.append(title);
         builder.append('\n');
@@ -944,11 +1001,16 @@ void Options::dumpOption(StringBuilder& builder, DumpLevel level, Options::ID id
 
 void Options::ensureOptionsAreCoherent()
 {
+    AllowUnfinalizedAccessScope scope;
     bool coherent = true;
     if (!(useLLInt() || useJIT())) {
         coherent = false;
         dataLog("INCOHERENT OPTIONS: at least one of useLLInt or useJIT must be true\n");
     }
+    if (useWebAssembly() && !(useWasmLLInt() || useBBQJIT())) {
+        coherent = false;
+        dataLog("INCOHERENT OPTIONS: at least one of useWasmLLInt or useBBQJIT must be true\n");
+    }
     if (!coherent)
         CRASH();
 }
