@@ -35,11 +35,8 @@
 #if USE(PTHREADS)
 
 #include <errno.h>
-#include <wtf/DataLog.h>
 #include <wtf/NeverDestroyed.h>
-#include <wtf/RawPointer.h>
 #include <wtf/StdLibExtras.h>
-#include <wtf/ThreadGroup.h>
 #include <wtf/ThreadingPrimitives.h>
 #include <wtf/WordLock.h>
 
@@ -66,6 +63,15 @@
 
 #endif
 
+#if OS(DARWIN)
+#include <mach/mach_traps.h>
+#include <mach/thread_switch.h>
+#endif
+
+#if OS(LINUX)
+#include <sys/syscall.h>
+#endif
+
 namespace WTF {
 
 static Lock globalSuspendLock;
@@ -185,6 +191,13 @@ void Thread::initializePlatformThreading()
 #endif
 }
 
+#if OS(LINUX)
+ThreadIdentifier Thread::currentID()
+{
+    return static_cast<ThreadIdentifier>(syscall(SYS_gettid));
+}
+#endif
+
 void Thread::initializeCurrentThreadEvenIfNonWTFCreated()
 {
 #if !OS(DARWIN)
@@ -201,7 +214,7 @@ static void* wtfThreadEntryPoint(void* context)
     return nullptr;
 }
 
-bool Thread::establishHandle(NewThreadContext* context)
+bool Thread::establishHandle(NewThreadContext* context, Optional<size_t> stackSize)
 {
     pthread_t threadHandle;
     pthread_attr_t attr;
@@ -209,6 +222,8 @@ bool Thread::establishHandle(NewThreadContext* context)
 #if HAVE(QOS_CLASSES)
     pthread_attr_set_qos_class_np(&attr, adjustedQOSClass(QOS_CLASS_USER_INITIATED), 0);
 #endif
+    if (stackSize)
+        pthread_attr_setstacksize(&attr, stackSize.value());
     int error = pthread_create(&threadHandle, &attr, wtfThreadEntryPoint, context);
     pthread_attr_destroy(&attr);
     if (error) {
@@ -556,7 +571,12 @@ void ThreadCondition::broadcast()
 
 void Thread::yield()
 {
+#if OS(DARWIN)
+    constexpr mach_msg_timeout_t timeoutInMS = 1;
+    thread_switch(MACH_PORT_NULL, SWITCH_OPTION_DEPRESS, timeoutInMS);
+#else
     sched_yield();
+#endif
 }
 
 } // namespace WTF
