@@ -525,7 +525,7 @@ LayoutRect MathOperator::paintGlyph(const RenderStyle& style, PaintInfo& info, c
     info.context().clip(clipBounds);
 
     GlyphBuffer buffer;
-    buffer.add(data.glyph, data.font, advanceWidthForGlyph(data));
+    buffer.add(data.glyph, *data.font, advanceWidthForGlyph(data));
     info.context().drawGlyphs(*data.font, buffer, 0, 1, origin, style.fontCascade().fontDescription().fontSmoothing());
 
     return glyphPaintRect;
@@ -729,7 +729,7 @@ void MathOperator::paint(const RenderStyle& style, PaintInfo& info, const Layout
         glyphData.glyph = m_variantGlyph;
 
     GlyphBuffer buffer;
-    buffer.add(glyphData.glyph, glyphData.font, advanceWidthForGlyph(glyphData));
+    buffer.add(glyphData.glyph, *glyphData.font, advanceWidthForGlyph(glyphData));
     LayoutPoint operatorTopLeft = paintOffset;
     FloatRect glyphBounds = boundsForGlyph(glyphData);
     LayoutPoint operatorOrigin { operatorTopLeft.x(), LayoutUnit(operatorTopLeft.y() - glyphBounds.y()) };
